/*-
 * Copyright (c) 1999 HIYAMA Takeshi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/*
 * Life Game for X
 * 
 * HIYAMA Takeshi
 */

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include "xlifegame.h"


main(argc, argv)
    int             argc;
    char           *argv[];
{
    int             a = 1, b = 0, c = 0, i, j, x, y, ***data, **data1,
                   *data2;
    char            key, *display_name = NULL, *colorname[] = {"blue", "cyan", "grey90", "grey70", "grey50"};
    FILE           *fi;
    KeySym          key_sym;

    for (i = 1; i < argc; i++) {
	if (!strncmp(argv[i], "-d",sizeof(char)*2))
	    display_name = argv[++i];
	else if(!strncmp(argv[i], "-w",sizeof(char)*2))
		wait=atoi(argv[++i]);
	else if(!strncmp(argv[i], "-x",sizeof(char)*2))
		xx=atoi(argv[++i]);
	else if(!strncmp(argv[i], "-y",sizeof(char)*2))
		yy=atoi(argv[++i]);
	else
	    OptionError(argv[i]);
    }

    data = (int ***) malloc(sizeof(int **) * 2);
    for (i = 0; i < 2; i++) {
	data[i] = (int **) malloc(sizeof(int *) * xx);
	for (j = 0; j < xx; j++)
	    data[i][j] = (int *) malloc(sizeof(int) * yy);
    }
    for (i = 0; i < xx; i++)
	for (j = 0; j < yy; j++)
	    data[0][i][j] = 0;
    for (i = 0; i < xx; i++)
	for (j = 0; j < yy; j++)
	    data[1][i][j] = 0;

    if (!(display = XOpenDisplay(display_name)))
	DisplayError(display_name);
    colmap = DefaultColormap(display, DefaultScreen(display));
    color[0] = XWhitePixel(display, DefaultScreen(display));
    color[6] = BlackPixel(display, DefaultScreen(display));
    for (i = 1; i < 6; i++) {
	XAllocNamedColor(display, colmap, colorname[i - 1], &col, &co);
	color[i] = col.pixel;
    }

    window = XCreateSimpleWindow(display, DefaultRootWindow(display), 0, 0, xx * size - 10, yy * size + 105, 2, color[6], color[0]);
    XMapWindow(display, window);
    button[0] = XCreateSimpleWindow(display, window, 10, yy * size + 10, 60, 30, 1, color[6], color[0]);
    button[1] = XCreateSimpleWindow(display, window, 100, yy * size + 10, 60, 30, 1, color[6], color[0]);
    button[2] = XCreateSimpleWindow(display, window, 20, yy * size + 60, 60, 30, 1, color[6], color[0]);
    button[3] = XCreateSimpleWindow(display, window, 90, yy * size + 60, 60, 30, 1, color[6], color[0]);
    for (i = 0; i < 4; i++) {
	XMapWindow(display, button[i]);
	XSelectInput(display, button[i], ButtonPressMask);
    }
    gc = XCreateGC(display, window, 0, 0);
    XSetBackground(display, gc, color[0]);
    XSetForeground(display, gc, color[6]);
    XSetFont(display, gc, XLoadFont(display, FONT_NAME));
    XStoreName(display, window, "LIFE GAME");
    XSelectInput(display, window, VisibilityChangeMask);
    XNextEvent(display, &event);
    XSelectInput(display, window, ButtonPressMask | KeyPressMask | ExposureMask);
    while (mode > -1) {
	/* Arranging Mode */
	do {
	    draw_window();
	    XNextEvent(display, &event);
	    if (event.type == ButtonPress) {
		if (event.xany.window == button[0]) {
		    quit;
		}
		else if (event.xany.window == button[1]) {
		    change_window_size();
		}
		else if (event.xany.window == button[2]) {
		    run;
		}
		else if (event.xany.window == button[3]) {
		    clear;
		}
		else if (event.xany.window == window) {
		    x = (event.xbutton.x + 5) / size;
		    y = (event.xbutton.y + 5) / size;
		    switch (event.xbutton.button) {
		    case 1:
			a=!data[b][x][y];
			if (click(x, y)) {
			    data[b][x][y] = a;
			    if (a) {
				set(x, y);
			    }
			    else {
				unset(x, y);
			    }
			    XFlush(display);
			}
			break;
		    case 2:
			clear;
			break;
		    case 3:
			run;
		    }
		}
	    }
	    else if (event.type == KeyPress) {
		key_event;
	    }
	    /* rewrite screen */
	    /* (when expose event occurerd */
	    else
		for (i = 1; i < xx - 1; i++)
		    for (j = 1; j < yy - 1; j++)
			if (data[b][i][j]) {
			    set(i, j);
			}
	} while (mode == 0);
	/* Running Mode */
	while (mode == 1) {
	    for (i = 1; i < xx - 1; i++)
		for (j = 1; j < yy - 1; j++) {
		    a = 0;
		    for (x = -1; x < 2; x++)
			for (y = -1; y < 2; y++)
			    a += data[b][i + x][j + y];
		    a -= data[b][i][j];
		    switch (a) {
		    case 2:
			data[!b][i][j] = data[b][i][j];
			break;
		    case 3:
			data[!b][i][j] = 1;
			break;
		    default:
			data[!b][i][j] = 0;
		    }
		    if (click(i, j)) {
			XSetForeground(display, gc, color[data[!b][i][j]]);
			XFillArc(display, window, gc, i * size - 5, j * size - 5, size, size, 0, 23040);
			if (data[!b][i][j] && size == 10) {
			    XSetForeground(display, gc, color[2]);
			    XFillArc(display, window, gc, i * size - 4, j * size - 4, 5, 5, 0, 23040);
			}
		    }
		}
	    b = !b;
	    draw_window();
	    usleep(wait);
	    if (XCheckMaskEvent(display, ButtonPressMask | KeyPressMask, &event)) {
		if (event.type == ButtonPress) {
		    if (event.xany.window == button[0])
			quit;
		    else if (event.xany.window == button[1])
			change_window_size();
		    else if (event.xany.window == button[2])
			stop;
		    else if (event.xany.window == window) {
			switch (event.xbutton.button) {
			case 3:
			    stop;
			    break;
			case 2:
			    quit;
			    break;
			}
		    }
		}
		else if (event.type == KeyPress) {
		    key_event;
		}
	    }
	}
    }
}
