/*-
 * Copyright (c) 1999 HIYAMA Takeshi
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#ifndef _XLIFEGAME_H_
#define _XLIFEGAME_H_

#define XLIFEGAME_VERSION "1.11"

#define FONT_NAME "fixed"

int             xx=60, yy=60, size=10, wait=10000, mode = 0;

Display        *display;
Window          window, button[6];
GC              gc;
XEvent          event;
unsigned long   color[7];
Colormap        colmap;
XColor          co, col;


void
OptionError(char *option)
{
    printf("   X Life Game version %s \n     by Takeshi Hiyama\n",XLIFEGAME_VERSION);
    if(strncmp(option, "-h", 3))
        printf("\nxlife:  bad command line option \"%s\"\n", option);
    printf("usage: xlife [-display <displayname>] [-wait microseconds (default 10000)]\n             [-x width (default 60)] [-y height (default 60)] [-h]\n");
    exit(1);
}

void
DisplayError(char *dname)
{
    printf("\nError: Can't open display: %s\n", dname);
    exit(1);
}


void
change_window_size()
{
    size = (size == 10) ? 2 : 10;
    if (size == 10) {
	XResizeWindow(display, window, xx * size -10, yy * size + 105);
	XMapWindow(display, button[2]);
	XMapWindow(display, button[3]);
	XMoveResizeWindow(display, button[0], 10, yy * size + 10, 60, 30);
	XMoveResizeWindow(display, button[1], 100, yy * size + 10, 60, 30);
    }
    else {
	XResizeWindow(display, window, xx * size-  10 , yy * size + 50);
	XUnmapWindow(display, button[2]);
	XUnmapWindow(display, button[3]);
	XMoveResizeWindow(display, button[0], 10, yy * size + 10, 30, 30);
	XMoveResizeWindow(display, button[1], 50, yy * size + 10, 30, 30);
    }
    XFlush(display);
}


void
draw_window()
{
    int             x1, y1;

    XSetForeground(display, gc, color[6]);

    if(size==10){
    XDrawRectangle(display, window, gc, 0, 0, xx * size - 11, yy * size - 11);
    XSetForeground(display, gc, color[5]);
    XDrawRectangle(display, window, gc, 2, 2, xx * size - 15, yy * size - 15);
    XSetForeground(display, gc, color[4]);
    XDrawRectangle(display, window, gc, 4, 4, xx * size - 19, yy * size - 19);
    XSetForeground(display, gc, color[3]);
    XDrawRectangle(display, window, gc, 7, 7, xx * size - 23, yy * size - 23);
}
else {
y1=yy*size-5;
XDrawLine(display, window, gc, 0, y1, xx*size, y1);
}

    XSetForeground(display, gc, color[6]);
    XDrawImageString(display, button[0], gc, 13, 15, "q uit", strlen("q uit"));
    XClearWindow(display, button[2]);
    XClearWindow(display, button[3]);
    XDrawImageString(display, button[3], gc, size, 17, "c lear", strlen("c lear"));

    XSetForeground(display, gc, color[0]);
    XFillRectangle(display, window, gc, xx * size - 300, yy * size + 10, 300, yy * size + 100);

    XSetForeground(display, gc, color[6]);
    if (size == 10) {
	XDrawImageString(display, button[1], gc, 2, 12, "m inimize", strlen("m inimize"));
	XDrawImageString(display, button[1], gc, 2, 25, "window", strlen("window"));

	XSetForeground(display, gc, color[6]);
	XDrawRectangle(display, window, gc, xx * size - 120, yy * size + 15, 100, 60);
	XDrawImageString(display, window, gc, xx * size - 70, yy * size + 45, "mouse", strlen("mouse"));
	x1 = xx * size - 90;
	y1 = yy * size + 15;
	XDrawLine(display, window, gc, x1, y1, x1, y1 + 60);
	y1 += 20;
	XDrawLine(display, window, gc, x1 - 30, y1, x1, y1);
	y1 += 20;
	XDrawLine(display, window, gc, x1 - 30, y1, x1, y1);
	y1 += 20;
	XDrawLine(display, window, gc, x1 - 30, y1, x1, y1);
	x1 -= 80;
	y1 -= 50;
	XDrawLine(display, window, gc, x1, y1, x1 + 60, y1);
	y1 += 20;
	XDrawLine(display, window, gc, x1, y1, x1 + 60, y1);
	y1 += 20;
	XDrawLine(display, window, gc, x1, y1, x1 + 60, y1);
	if (!mode) {
	    XDrawImageString(display, button[2], gc, 15, 17, "r un", strlen("r un"));
	    x1 = xx * size - 230;
	    y1 = yy * size + 27;
	    XDrawImageString(display, window, gc, x1, y1, "run   : r", strlen("run   : r"));
	    y1 += 20;
	    XDrawImageString(display, window, gc, x1, y1, "clear : c", strlen("clear : c"));
	    y1 += 20;
	    XDrawImageString(display, window, gc, x1, y1, "(un)set", strlen("(un)set"));
	}
	else {
	    XDrawImageString(display, button[2], gc, 13, 17, "s top", strlen("s top"));
	    XDrawLine(display, button[3], gc, 0, 0, 50, 30);
	    XDrawLine(display, button[3], gc, 0, 30, 50, 0);

	    x1 = xx * size - 230;
	    y1 = yy * size + 47;
	    XDrawImageString(display, window, gc, x1, y1, "stop  : s", strlen("stop  : s"));
	    y1 += 20;
	    XDrawImageString(display, window, gc, x1, y1, "quit  : q", strlen("quit  : q"));
	}
    }
    else {
	XDrawImageString(display, button[1], gc, 2, 12, "m axi-", strlen("m axi-"));
	XDrawImageString(display, button[1], gc, 2, 25, "mize", strlen("mize"));

    }
    XFlush(display);
}


#define quit		mode=-1
#define run		mode=1
#define clear		for (i = 1; i < xx - 1; i++)\
 			for (j = 1; j < yy - 1; j++) \
			data[b][i][j] = 0;\
 			XClearWindow(display, window);\
 			XFlush(display);
#define stop		mode=0

#define click(x,y)	((x < xx-1 && x > 0 && y < yy-1 && y > 0) ? 1 : 0)

void
set(int x, int y)
{
    XSetForeground(display, gc, color[1]);
    XFillArc(display, window, gc, x * size - 5, y * size - 5, size, size, 0, 23040);
    if(size==10){
    XSetForeground(display, gc, color[2]);
    XFillArc(display, window, gc, x * size - 4, y * size - 4, 5, 5, 0, 23040);
}
}

void
unset(int x, int y)
{
    XSetForeground(display, gc, color[0]);
    XFillRectangle(display, window, gc, x * size - 5, y * size - 5, size,size);
}

#define key_event  XLookupString(&event.xkey, &key, 1, &key_sym, NULL);\
    switch (key) {\
    case 'q': quit; break;\
    case 'r': run; break;\
    case 'c': if(!mode)clear; break;\
    case 's': stop; break;\
    case 'm': change_window_size();}

#endif				/* _XLIFEGAME_H_ */
